md5 : 2347a89d251a2eb3c43b21dfd7362cf7
name : _11_PerlOMP_1D_Array_TO_1D_FLOAT_ARRAY_t_2347
version : ""
language : C
language_id : C
installed : 0
date_compiled : Thu Sep  5 08:43:47 2024
inline_version : 0.86
ILSM : %
    module : Inline::C
    suffix : so
    type : compiled
Config : %
    apiversion : ?
    archname : x86_64-linux
    cc : cc
    ccflags : -fwrapv -fno-strict-aliasing -pipe -fstack-protector-strong -I/usr/local/include -D_LARGEFILE_SOURCE -D_FILE_OFFSET_BITS=64 -D_FORTIFY_SOURCE=2
    ld : cc
    osname : linux
    osvers : 6.1.0-18-amd64
    so : so
    version : 5.40.0
