md5 : 18a3cf04720d42c81ea50d060326f8b7
name : _02_PerlOMP_UPDATE_WITH_ENV__NUM_THREADS_t_18a3
version : ""
language : C
language_id : C
installed : 0
date_compiled : Thu Sep  5 08:43:25 2024
inline_version : 0.86
ILSM : %
    module : Inline::C
    suffix : so
    type : compiled
Config : %
    apiversion : ?
    archname : x86_64-linux
    cc : cc
    ccflags : -fwrapv -fno-strict-aliasing -pipe -fstack-protector-strong -I/usr/local/include -D_LARGEFILE_SOURCE -D_FILE_OFFSET_BITS=64 -D_FORTIFY_SOURCE=2
    ld : cc
    osname : linux
    osvers : 6.1.0-18-amd64
    so : so
    version : 5.40.0
